/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2009 Canonical Services Ltd (www.canonical.com)
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"
#include <webkit/webkit.h>
#include "oauth.h"
#include "u1-music-store.h"
#include "u1-marshal.h"

#define U1_STORE_URL "http://one.ubuntu.com/music/login"
#define U1_PREVIEW_JAVASCRIPT "" \
	"(function() {" \
	"  // capture the preview" \
	"  if (playTrack) {" \
	"    playTrack = function(file, title) {" \
	"      window.status = \"u1preview:::\" + file + \":::\" + title;" \
	"    }" \
	"  }" \
	"  // and hide the flashplayer"\
	"  var ply = document.getElementById(\"flashPlayer\");" \
	"  if (ply) {" \
	"    ply.parentNode.removeChild(ply);" \
	"  }" \
	"})()"
#define U1_LIBRARY_OVERRIDE_SCRIPT "" \
	"(function(){" \
        "  var lnks = document.querySelectorAll(\"a\");" \
	"  for (var i=0; i<lnks.length; i++) {" \
	"    if (lnks[i].href.indexOf(\"library.aspx\") != -1) {" \
	"      lnks[i].href = \"%s\";" \
	"    }" \
	"  }" \
	"})()"
#define U1_DEFAULT_ERROR_PAGE "<html><body>Could not load Music Store</body></html>"

struct _U1MusicStorePrivate {
	GtkWidget *web_viewer;

	GdkCursor *busy_cursor;
	GdkCursor *normal_cursor;
};

G_DEFINE_TYPE(U1MusicStore, u1_music_store, GTK_TYPE_VBOX)

enum {
	PREVIEW_MP3_SIGNAL,
	LAST_SIGNAL
};

static guint u1_music_store_signals[LAST_SIGNAL] = { 0, };

static void
u1_music_store_finalize (GObject *object)
{
	U1MusicStore *music_store = U1_MUSIC_STORE (object);

	if (music_store->priv != NULL) {
		g_free (music_store->priv);
		music_store->priv = NULL;
	}

	G_OBJECT_CLASS (u1_music_store_parent_class)->finalize (object);
}

static void
u1_music_store_class_init (U1MusicStoreClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	/* Signals */
	u1_music_store_signals[PREVIEW_MP3_SIGNAL] = g_signal_new ("preview-mp3",
								   G_TYPE_FROM_CLASS (klass),
								   (GSignalFlags) G_SIGNAL_RUN_LAST,
								   G_STRUCT_OFFSET (U1MusicStoreClass, preview_mp3),
								   NULL,
								   NULL,
								   _u1_marshal_VOID__STRING_STRING,
								   G_TYPE_NONE, 2,
								   G_TYPE_STRING, G_TYPE_STRING);
	object_class->finalize = u1_music_store_finalize;
}

static gboolean
navigation_requested_cb (WebKitWebView *web_view,
			 WebKitWebFrame *frame,
			 WebKitNetworkRequest *request,
			 WebKitWebNavigationAction *navigation_action,
			 WebKitWebPolicyDecision *policy_decision,
			 gpointer user_data)
{
	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);

	/* Change the cursor when loading */
	if (music_store->priv->normal_cursor)
		gdk_cursor_unref (music_store->priv->normal_cursor);
	music_store->priv->normal_cursor = gdk_window_get_cursor (GTK_WIDGET (web_view)->window);
	gdk_cursor_ref (music_store->priv->normal_cursor);

	if (music_store->priv->busy_cursor == NULL)
		music_store->priv->busy_cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_WATCH);

	gdk_window_set_cursor (GTK_WIDGET (web_view)->window, music_store->priv->busy_cursor);

	/* Inform webkit to continue with the loading */
	webkit_web_policy_decision_use (policy_decision);

	return TRUE;
}
			 
static void
load_finished_cb (WebKitWebView *web_view, WebKitWebFrame *frame, gpointer user_data)
{
	const gchar *library_override;
	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);

	/* Inject some JavaScript to enable previews and hide the Flash player */
	webkit_web_view_execute_script (web_view, U1_PREVIEW_JAVASCRIPT);

	/* Allow us to specifically override the library page links */
	library_override = g_getenv ("U1MUSICLIBRARYURL");
	if (library_override != NULL) {
		gchar *script = g_strdup_printf (U1_LIBRARY_OVERRIDE_SCRIPT, library_override);

		webkit_web_view_execute_script (web_view, script);
		g_free (script);
	}

	gdk_window_set_cursor (GTK_WIDGET (web_view)->window, music_store->priv->normal_cursor);
}

static void
status_bar_text_changed_cb (WebKitWebView *web_view, const gchar *text, gpointer user_data)
{
	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);

	if (g_str_has_prefix (text, "u1preview:::")) {
		gchar **args;

		args = g_strsplit (text, ":::", 0);
		if (args != NULL) {
			g_signal_emit (music_store, u1_music_store_signals[PREVIEW_MP3_SIGNAL], 0, args[1], args[2]);
			g_strfreev (args);
		}
	}
}

static gboolean
load_error_cb (WebKitWebView *web_view, WebKitWebFrame *frame, const gchar *uri, GError *error, gpointer user_data)
{
	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);

	webkit_web_view_load_string (web_view, U1_DEFAULT_ERROR_PAGE, "text/html", "utf-8", "file:///");

	return TRUE;
}

static void
u1_music_store_init (U1MusicStore *music_store)
{
	GtkWidget *scroll;
	const gchar *url_to_use;
	gchar *real_url;
	gchar *oauth_token, *oauth_token_secret;

	music_store->priv = g_new0 (U1MusicStorePrivate, 1);

	/* Create web viewer object */
	scroll = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
	gtk_widget_show (scroll);

	music_store->priv->web_viewer = webkit_web_view_new ();
	g_object_set (G_OBJECT (webkit_web_view_get_settings (WEBKIT_WEB_VIEW (music_store->priv->web_viewer))),
		      "user-agent", "U1MusicStore widget (" VERSION ")",
		      NULL);

	g_signal_connect (G_OBJECT (music_store->priv->web_viewer), "navigation-policy-decision-requested",
			  G_CALLBACK (navigation_requested_cb), music_store);
	g_signal_connect (G_OBJECT (music_store->priv->web_viewer), "load-finished",
			  G_CALLBACK (load_finished_cb), music_store);
	g_signal_connect (G_OBJECT (music_store->priv->web_viewer), "status-bar-text-changed",
			  G_CALLBACK (status_bar_text_changed_cb), music_store);
	g_signal_connect (G_OBJECT (music_store->priv->web_viewer), "load-error",
			  G_CALLBACK (load_error_cb), music_store);
	gtk_widget_show (music_store->priv->web_viewer);
	gtk_container_add (GTK_CONTAINER (scroll), music_store->priv->web_viewer);

	/* If U1MUSICSTOREURL is defined, use that instead of the real URL */
	if (! (url_to_use = g_getenv ("U1MUSICSTOREURL")))
		url_to_use = U1_STORE_URL;

	/* OAUth sign the URL */
	if (g_str_has_prefix (url_to_use, "http://localhost") || g_str_has_prefix (url_to_use, "http://127.0.0.1")) {
		const gchar *oauthfile;

		oauthfile = g_getenv ("OAUTHKEYFILE");
		if (oauthfile != NULL) {
			gchar *oauthstring;
			gsize len;

			if (g_file_get_contents (oauthfile, &oauthstring, &len, NULL)) {
				gchar **parser;

				parser = g_strsplit (oauthstring, "&", 2);
				oauth_token = g_strdup (parser[0]);
				oauth_token_secret = g_strdup (parser[1]);

				g_strfreev (parser);
			} else {
				oauth_token = g_strdup ("fail");
				oauth_token_secret = g_strdup ("fail");
			}
		} else {
			webkit_web_view_load_string (music_store->priv->web_viewer,
						     "Fail! If you specify U1MUSICSTOREURL you must "
						     "also specify OAUTHKEYFILE as a file with OAuth keys in it",
						     "text/html", "utf-8", "file:///");
		}
	} else {
		oauth_token = g_strdup ("fail");
		oauth_token_secret = g_strdup ("fail");
	}

	real_url = oauth_sign_url2 (url_to_use, NULL, OA_HMAC, "GET",
				    "ubuntuone", "hammertime",
				    oauth_token, oauth_token_secret);
	webkit_web_view_open (WEBKIT_WEB_VIEW (music_store->priv->web_viewer), real_url);

	gtk_box_pack_start (GTK_BOX (music_store), scroll, TRUE, TRUE, 3);

	/* Free memory */
	g_free (real_url);
	g_free (oauth_token);
	g_free (oauth_token_secret);
}

/**
 * u1_music_store_new:
 *
 * Create a new #U1MusicStore widget.
 *
 * Return value: the newly created widget.
 */
GtkWidget *
u1_music_store_new (void)
{
	U1MusicStore *music_store;

	music_store = g_object_new (U1_TYPE_MUSIC_STORE, NULL);

	return (GtkWidget *) music_store;
}
